/*-------------------------------------------------------------------------

project: genFamMem - maintaining families of requirements

author: Jan Bredereke
copyright: Jan Bredereke, 2000
$Id: filterspec.h,v 1.8 2001/02/09 11:21:27 brederek Exp $

file: filterspec.h - header file: filter a specification, deleting and
		     editing lines as appropriate

If NDEBUG is defined, the internal assertions are not checked anymore.

-------------------------------------------------------------------------*/

#ifndef FILTERSPEC_H
#define FILTERSPEC_H

#include <stdlib.h>
#include <stdio.h>
#include "linerange.h"

/*-------------------------------------------------------------------------
The state of the filtering, as far as it needs to be accessed globally 
by the lexer
-------------------------------------------------------------------------*/

struct filStateType
{   int currLineNo; /* current line number */
    int printing; /* 1: currently printing lines */
                  /* 0: replacing lines by suitable comments */
    int inheritArea; /* 1: currently in "inherit" area, do editing */
                     /* 0: currently not in "inherit" area, don't do editing */
    int parentsArea; /* 1: currently in "parents" area, do editing */
                     /* 0: currently not in "parents" area, don't do editing */
    int endfeaturesFound; /* 1: current line contains an "\endfeatures",
    				possibly append generated family member
				feature chapter body */
			  /* 0: otherwise */
    int pendingEmptyLinesToDelete; /* the number of empty lines which we 
    				      should delete to make up for inserted
				      lines */
};
extern struct filStateType filState;

/*-------------------------------------------------------------------------
The functions for external use
-------------------------------------------------------------------------*/

/* copy the file literally to stdout, except for deleted lines, and
   except for editing of "inherit" areas */
extern void
filFilterSpec(char* inFileName, char language);

/* call-back function from the lexer to store text to memory from an
  "inherit" or "parents" area */
extern void
filSaveBuffer(char* text);

/* call-back function from the lexer to switch to next line in an
  "inherit" or "parents" area */
extern void
filNextBufferLine();

#endif /* FILTERSPEC_H */
