/*-------------------------------------------------------------------------

project: genFamMem - maintaining families of requirements

author: Jan Bredereke
copyright: Jan Bredereke, 2000
$Id: genidentifier.c,v 1.4 2001/01/29 10:35:15 brederek Exp $

file: genidentifier.c - generate unique identifiers

An extension of identifier.c. Provides a function to generate a
unique identifier from an identifier. The generated identifier
contains a "." and thus cannot create clashes with parsed
identifiers.

-------------------------------------------------------------------------*/

#include <string.h>
#include <assert.h>
#include <stdio.h>
#include "genidentifier.h"

/*-------------------------------------------------------------------------
The local state
-------------------------------------------------------------------------*/

static int nextGenNum = 1; /* Next number used for generating a unique
			      identifier. */


/*-------------------------------------------------------------------------
The functions for internal use
-------------------------------------------------------------------------*/

/* assertMem() */
/* check that an expression is true; if not, complain about insufficient
   memory and exit with code 2 */
#define assertMem(expr) (expr ? 0 : assertMemFail())
static int
assertMemFail()
{   fprintf(stderr, "\n\nCannot allocate further memory for the table of identifiers.\nExiting...\n");
    exit(2);
}

/* complain about insufficient range of internal identifier numbers
   and exit with code 2 */
static int
assertIdFail()
{   fprintf(stderr, "\n\nCannot generate more internal identifier numbers (than 99999).\nExiting...\n");
    exit(2);
}

/*-------------------------------------------------------------------------
The functions for external use
-------------------------------------------------------------------------*/

/* generate a unique identifier from a provided identifier */
/* returns: a non-negative id number on success */
/* exits with an error message and code 2 if there is no more space in the */
/*          internal table or if there are no more internal identifier */
/*          numbers */
/* The name generated may be obtained by idId2Identifier(). */
extern int
gidGenUniqueIdentifier(char *str)
{   char *identifier; /* temporary copy of resulting string */
    int strLen;  /* length of resulting string, without terminating \0 */
    int retCode;

    assert(str);
    strLen = strlen(str)+7;
    identifier = (char *) malloc((strLen+1) * sizeof(char));
    assertMem(identifier);
    strcpy(identifier, str);
    /* Make sure that we don't append more than there is space for: */
    if(nextGenNum > 99999)
        assertIdFail();
    /* append a dot and a unique number: */
    sprintf(&identifier[strLen-7], ".%.5i", nextGenNum++);
    retCode = idIdentifier2Id(identifier);
    free(identifier);
    return(retCode);
}

