/*-------------------------------------------------------------------------

project: genFamMem - maintaining families of requirements

author: Jan Bredereke
copyright: Jan Bredereke, 2000
$Id: global.h,v 1.46 2001/02/20 18:02:19 brederek Exp $

file: global.h - global declarations and definitions for the generator program

-------------------------------------------------------------------------*/

#include "relation.h"
#include "linerange.h"

/* name of this program */
#define PROGNAME "genFamMem"
/* version */
#define PROGVERSION "2.0.3"

/* exit codes */
/* OK */
#define EXITOK 0
/* some syntax error found */
#define EXITSYNERR 1
/* illegal parameters specified */
#define EXITPARAMS 2
/* not sufficient system resources available */
#define EXITRESOURCES 3

/* input languages */
#define LANGUAGE_Z 1
#define LANGUAGE_CSPZ 2
#define LANGUAGE_CSPOZ 3

#define LANGUAGE_DEFAULT LANGUAGE_CSPOZ

/* kind of uses-hierarchy requirested */
/* no uses-hierarchy requested */
#define UHIERNO 0
/* uses-hierarchy of all sections requested */
#define UHIERALL 1
/* uses-hierarchy for one family member req. */
#define UHIERFAMMEM 2
/* uses-hierarchy for one feature requested */
#define UHIERFEAT 3


/* the command line options of the program, as far as they need
   to be accessed globally */
struct progOptsType
{   char *inFileName; /* name of input file */
    char debug; /* 0: no debug output, otherwise: debug output */
    char markup; /* 1: do mark-up only and print it, 
                    otherwise: normal operation */
    char language; /* as defined above in LANGUAGE_* */
    int selectedFamilyMember; /* the family member selected on the 
    				 command line (also for --uhier-familymember) */
    int selectedFeature; /* the feature selected by the 
    			    option --uhier-feature */
    char usesHier; /* UHIERNO: no uses-hierarchy requested */
                   /* UHIERALL: uses-hierarchy of all sections requested */
                   /* UHIERFAMMEM: uses-hierarchy for one family member req. */
                   /* UHIERFEAT: uses-hierarchy for one feature requested */
    char ascii; /* 1: uses-hierarchy output in ASCII format */
    		/* 0: uses-hierarchy output in daVinci format */
    char onlyFeature; /* abstract sections to features in uses-hierarchy */
};
extern struct progOptsType progOpts;

/* the diverse relations on identifiers, globally accessed by the
   different lex/yacc parser stages */
struct idRelsType
{   relSetH features;		/* features of the family */
    relSetH featureEnds;	/* features with a feature_end construct */
    relSetH anonymousFeatures;	/* features which are anonymous */
    relSetH familyMembers;	/* family members of the family */
    relSetH familyMemberEnds;	/* family members with a familymember_end 
    				   construct */
    relSetH nonPureFamilyMembers; /* non-"pure" family members with a feature 
    				    chapter body */
    relSetH sections;		/* sections of the family */
    relRelationH famMem2FeatureList; /* relation: the feature lists of the 
                                        family members */
    relRelationH feature2Section; /* relation: the sections of the features */
    relSetH defaultSections;    /* those sections which are default sections */
    relSetH collectSections;    /* those sections which are collect sections */
    relSetH sectionEnds;	/* sections with a section_end construct */
    relRelationH sectionParents; /* relation: the parents of the sections */
    relRelation3H sectionClass2Inherits; /* relation: the inherited classes of
    				    the classes of the sections */
    relRelation3H sectionClass2DefaultProperties; /* relation: the same as
    				    above, but only those that are inherited
				    in a "default properties" construct */
    relRelationH remove;	/* relation: the sections removed by a feature*/
    relSetH selectedFeatures; /* the features selected by the selected 
    				 family member */
    relSetH selectedRemove; /* the sections removed by the selected family 
    			       member */
    relSetH selectedSections; /* the sections selected by the selected 
    				 family member; or the sections for the
				 uses-hierarchy */
    relRelationH selectedSectionParents; /* relation: the selected parents
    					    of the selected sections */
    relRelationH selectedSectionDeletedParents; /* relation: the deleted parents
    					    of the selected sections */
    relRelationH section2Class; /* relation: the classes of the sections */
    relRelationH feature2FirstLine; /* first line of the feature, or 
    				       zero if anonymous feature */
    relRelationH feature2LastLine; /* last line of the feature, or 
				      -1 if at end-of-input */
    relRelationH famMem2FirstLine; /* first line of the family member */
    relRelationH famMem2LastLine; /* last line of the family member, or
				     -1 if at end-of-input */
    relRelationH section2FirstLine; /* first line of the section */
    relRelationH section2LastLine;  /* last line of the section, 
				       or -1 if at end-of-input */
    relRelation3H sectionClass2FirstLine; /* first line of the class of a
    					    section */
					  /* NOTE: This is currently set,
					     but not used anywhere */
    relRelation3H sectionClass2LastLine; /* last line of the class of a section,
				       or -1 if at end-of-input */
					  /* NOTE: This is currently set,
					     but not used anywhere */
    relRelation3H sectionClassInherit2FirstLine; /* first line of the 
    					inheritance etc. part of a class 
					in a section, if any */
    relRelation3H sectionClassInherit2LastLine; /* last line of the above */
    relRelation3H selectedSectionClassDeletedDefaultProperties;
    				/* for each class of each selected section, the
				   classes deleted in its "inherit" area */
    relRelationH sectionParents2FirstLine; /* first line of the parents part
    					of a section, if any */
    relRelationH sectionParents2LastLine; /* last line of the above */
    relSetH deletedFeatures; /* the features deleted because not selected */
    relSetH deletedFamilyMembers; /* the family members deleted because not
    				     selected */
    linLineRangeH deletedLineRanges; /* the line ranges deleted because not
    				        selected */
    linLineRangeH parentsLineRanges; /* the "parents" area line ranges of the
    				        selected sections */
    linLineRangeH inheritLineRanges; /* the "inherit" area line ranges of the
    				        selected sections */
    relRelationH usesHierarchy; /* relation: the uses-hierarchy to print */
    relSetH usesHierarchyRoots; /* the root nodes of the uses-hierarchy */
    relRelationH usesHierarchyLevel; /* relation: the level of the nodes in
    				        the uses-hierarchy */
    int usesHierarchyDepth; /* the max. level of the uses-hierarchy */
};
extern struct idRelsType idRels;

