/*-------------------------------------------------------------------------

project: genFamMem - maintaining families of requirements

author: Jan Bredereke
copyright: Jan Bredereke, 2000
$Id: identifier.h,v 1.4 2001/01/26 14:14:47 brederek Exp $

file: identifier.h - header file: conversion of identifiers to integer
		     ID numbers

The ID numbers are generated as we go on and should not be interpreted.

-------------------------------------------------------------------------*/

#ifndef IDENTIFIER_H
#define IDENTIFIER_H

/*-------------------------------------------------------------------------
The constants for external use
-------------------------------------------------------------------------*/

/* return codes */
/* ERROR */
#define IDERR	  -1

/*-------------------------------------------------------------------------
The types for external use
-------------------------------------------------------------------------*/

/* Identifiers are the usual null-terminated character strings. */

/* Ids are non-negative integers. */

/*-------------------------------------------------------------------------
The functions for external use
-------------------------------------------------------------------------*/

/* convert an identifier into an id number */
/* returns: a non-negative id number on success */
/* exits with an error message and code 2 if there is no more space in the */
/*          internal table */
/* Same identifiers always get the same id number. */
/* Identifiers may be arbitrary long. */
extern int
idIdentifier2Id(char *str);

/* convert an id number into an identifier */
/* returns: a pointer to the identifier on success */
/*          NULL if the id number has not yet been allocated */
extern char *
idId2Identifier(int id);

/* take two identifiers, concatenate their string values, and return the
   id number of the result */
/* returns: a non-negative id number on success */
/*          IDERR if some of the id numbers have not yet been allocated */
/* exits with an error message and code 2 if there is no more space in the */
/*          internal table */
extern int
idConcatIds2Id(int id1, int id2);

#endif /* IDENTIFIER_H */
