/*-------------------------------------------------------------------------

project: genFamMem - maintaining families of requirements

author: Jan Bredereke
copyright: Jan Bredereke, 2000
$Id: printuseshier.c,v 1.7 2001/02/01 18:08:54 brederek Exp $

file: printuseshier.c - print the uses-hierarchy in the requested format

If NDEBUG is defined, the internal assertions are not checked anymore.

-------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "printuseshier.h"
#include "relation.h"

/*-------------------------------------------------------------------------
The local variables for internal use
-------------------------------------------------------------------------*/

relSetH nodeIsPrinted = NULL; /* the set of nodes already printed */

/*-------------------------------------------------------------------------
The local functions for internal use
-------------------------------------------------------------------------*/

/* print indent string */
static void
lPrintIndentString(int indentLevel)
{   int idx;

    for(idx = 0; idx < indentLevel; idx++)
    {   printf("    ");
    }
}

/* print the uses-hierarchy */
static void
lPrintUsesHierarchy(int currNode, int currLevel, int indentLevel)
{   int retCode;
    relSetIt setIt, setIt2; /* iterators over sets */
    relSetH tmpSet, tmpSet2 = NULL; /* auxiliary sets */
    char *continueStr = NULL; /* continuation string when printing */

    tmpSet = relNewSet();
    tmpSet2 = relNewSet();

    /* print header for this node */
    printf("l(\"%s\", n(\"class\", [a(\"OBJECT\",\"%s\")",
           idId2Identifier(currNode),
           idId2Identifier(currNode));
    /* maybe add a graphical attribute for this node */
    if(!progOpts.onlyFeature)
    {   printf(",a(\"COLOR\",\"%s\")",
	       ((relIsMember(idRels.defaultSections, currNode) == RELOK)
	        ? DAVINCI_BOXCOLOR_DEFPROP
		: DAVINCI_BOXCOLOR_NORM ));
        if(relIsMember(idRels.collectSections, currNode) == RELOK)
            printf(",a(\"BORDER\",\"double\")");
    }
    else
    {   printf(",a(\"COLOR\",\"%s\")", DAVINCI_BOXCOLOR_NORM);
    }
    printf("], [\n");
    /* mark node as printed */
    retCode = relAddMember(nodeIsPrinted, currNode);
    assert(retCode == RELOK);
    /* loop over all predecessor nodes */
    relApply(tmpSet, idRels.usesHierarchy, currNode);
    for(relInitSetIt(&setIt, tmpSet);
        relMoreSetIt(&setIt);
	relNextSetIt(&setIt)
       )
    {   /* print indent string */
        lPrintIndentString(indentLevel);
	/* prepare continuation string */
	if(!relLastSetIt(&setIt))
	    continueStr = ")),";
	else
	    continueStr = "))";
	/* print the edge to the predecessor node */
	printf("l(\"Edge %s -> %s\",e(\"parent\", [],\n",
	       idId2Identifier(currNode),
	       idId2Identifier(relCurrSetIt(&setIt))
	       );
	/* print the indent string */
        lPrintIndentString(indentLevel);
	/* if the predecessor node is at the exactly next level */
	retCode = relApply(tmpSet2,
	                   idRels.usesHierarchyLevel,
			   relCurrSetIt(&setIt));
	assert(retCode == RELOK);
	assert(relCardSet(tmpSet2) == 1);
	relInitSetIt(&setIt2, tmpSet2);
	if(relCurrSetIt(&setIt2) == currLevel + 1)
	{   /* if predecessor not yet printed, do it */
	    if(relIsMember(nodeIsPrinted, relCurrSetIt(&setIt)) == RELCANTDO)
	    {   printf("  ");
		lPrintUsesHierarchy(relCurrSetIt(&setIt),
				    currLevel + 1,
				    indentLevel + 1);
		lPrintIndentString(indentLevel);
		printf("%s\n", continueStr);
	    }
	    else
	    {   /* else print a link only */
	        printf("    r(\"%s\")%s\n",
		       idId2Identifier(relCurrSetIt(&setIt)),
		       continueStr);
	    }
	}
	else
	{   /* else print a link only */
	    printf("    r(\"%s\")%s\n",
		   idId2Identifier(relCurrSetIt(&setIt)),
		   continueStr);
	}
    }
    /* print closing for this node */
    lPrintIndentString(indentLevel);
    printf("]))\n");

    /* free allocated dynamic memory */

    relFreeSet(tmpSet);
    relFreeSet(tmpSet2);
}

/*-------------------------------------------------------------------------
The functions for external use
-------------------------------------------------------------------------*/

/* print the uses-hierarchy */
void
uhPrintUsesHierarchy()
{   relSetIt setIt; /* an iterator over sets */

    nodeIsPrinted = relNewSet();

    printf("[\n");
    for(relInitSetIt(&setIt, idRels.usesHierarchyRoots);
        relMoreSetIt(&setIt);
	relNextSetIt(&setIt)
       )
    {   printf("  ");
        lPrintUsesHierarchy(relCurrSetIt(&setIt) , 0, 1);
    }
    printf("]\n");

    /* free allocated dynamic memory */

    relFreeSet(nodeIsPrinted);

}

